/*
* Copyright (c) Contributors, http://opensimulator.org/
* See CONTRIBUTORS.TXT for a full list of copyright holders.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions are met:
*     * Redistributions of source code must retain the above copyright
*       notice, this list of conditions and the following disclaimer.
*     * Redistributions in binary form must reproduce the above copyright
*       notice, this list of conditions and the following disclaimer in the
*       documentation and/or other materials provided with the distribution.
*     * Neither the name of the OpenSim Project nor the
*       names of its contributors may be used to endorse or promote products
*       derived from this software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE DEVELOPERS AS IS AND ANY
* EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
* DISCLAIMED. IN NO EVENT SHALL THE CONTRIBUTORS BE LIABLE FOR ANY
* DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
* LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
* ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
* (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
* 
*/

using System;
using System.Collections.Generic;
using System.Data;
using libsecondlife;
using MySql.Data.MySqlClient;
using OpenSim.Framework.Console;

using Opensim.Framework.Persistence;
using Opensim.Framework.Persistence.mappings;
using Opensim.Framework.Persistence.interfaces;

using NHibernate;


namespace OpenSim.Framework.Data.Hibernate
{
    internal class AssetData : IAssetProvider
    {
        private Manager _dbConnection;

        private IDaoFactory _daoFactory;
        
        private IAssetsDao _assetsDao;

        #region IAssetProvider Members

        private void UpgradeAssetsTable(string oldVersion)
        {
            // null as the version, indicates that the table didn't exist
            if (oldVersion == null)
            {
                MainLog.Instance.Notice("ASSETS", "Creating new database tables");
                _dbConnection.ExecuteResourceSql("CreateAssetsTable.sql");
                return;
            }
        }

        /// <summary>
        /// Ensure that the assets related tables exists and are at the latest version
        /// </summary>
        private void TestTables()
        {
            Dictionary<string, string> tableList = new Dictionary<string, string>();

            tableList["assets"] = null;
            _dbConnection.GetTableVersion(tableList);

            UpgradeAssetsTable(tableList["assets"]);
        }

        public AssetBase FetchAsset(LLUUID assetID)
        {
            AssetBase asset = null;
            lock (_dbConnection)
            {
                MySqlCommand cmd =
                    new MySqlCommand(
                        "SELECT name, description, assetType, invType, local, temporary, data FROM assets WHERE id=?id",
                        _dbConnection.Connection);
                MySqlParameter p = cmd.Parameters.Add("?id", MySqlDbType.Binary, 16);
                p.Value = assetID.GetBytes();
                
                try
                {
                    using (MySqlDataReader dbReader = cmd.ExecuteReader(CommandBehavior.SingleRow))
                    {
                        if (dbReader.Read())
                        {
                            asset = new AssetBase();
                            asset.Data = (byte[]) dbReader["data"];
                            asset.Description = (string) dbReader["description"];
                            asset.FullID = assetID;
                            asset.InvType = (sbyte) dbReader["invType"];
                            asset.Local = ((sbyte) dbReader["local"]) != 0 ? true : false;
                            asset.Name = (string) dbReader["name"];
                            asset.Type = (sbyte) dbReader["assetType"];
                        }
                        dbReader.Close();
                        cmd.Dispose();
                    }
                }
                catch (Exception e)
                {
                    MainLog.Instance.Error(
                        "ASSETS", "MySql failure fetching asset {0}" + Environment.NewLine + e.ToString()
                       + Environment.NewLine + "Attempting reconnection", assetID);
                    _dbConnection.Reconnect();
                }
            }
            return asset;
        }

        public void CreateAsset(AssetBase asset)
        {        
    
            // TODO - Database lock needed???
            // TODO - Why a replace??? Easy method for insertOrUpdate - If exists, update, if not, insert

            // Impalah - Using Hibernate
            bool f_upd = false;

            // Create asset
            Assets dbasset;
            try
            {
                // Check if asset exist in DB
                dbasset = _assetsDao.GetById(asset.FullID.ToString(), true);
                f_upd = true;
            }
            catch (NHibernate.ObjectNotFoundException nfe)
            {
                // Object does not exist
                dbasset = new Assets();
                dbasset.Uuid = asset.FullID.ToString();
            }
            
            // If does not exist, create new

            dbasset.Name = asset.Name;
            dbasset.Description = asset.Description;
            dbasset.AssetType =  (byte)asset.Type;
            dbasset.InvType = (byte)asset.InvType;
            dbasset.Local = (byte)(asset.Local==true?1:0);
            dbasset.Temporary = (byte)(asset.Temporary==true?1:0);
            dbasset.Data = asset.Data;

            // Save or update
            if (f_upd)
                _assetsDao.Update(dbasset);
            else
                _assetsDao.Save(dbasset);
    

        }

        public void UpdateAsset(AssetBase asset)
        {
            CreateAsset(asset);
        }

        public bool ExistsAsset(LLUUID uuid)
        {
            throw new Exception("The method or operation is not implemented.");
        }

        /// <summary>
        /// All writes are immediately commited to the database, so this is a no-op
        /// </summary>
        public void CommitAssets()
        {
            // commit changes at the end
            _assetsDao.CommitChanges();
        }

        #endregion

        #region IPlugin Members

        public void Initialise()
        {
            //IniFile GridDataMySqlFile = new IniFile("mysql_connection.ini");
            //string hostname = GridDataMySqlFile.ParseFileReadValue("hostname");
            //string database = GridDataMySqlFile.ParseFileReadValue("database");
            //string username = GridDataMySqlFile.ParseFileReadValue("username");
            //string password = GridDataMySqlFile.ParseFileReadValue("password");
            //string pooling = GridDataMySqlFile.ParseFileReadValue("pooling");
            //string port = GridDataMySqlFile.ParseFileReadValue("port");

            //_dbConnection = new Manager(hostname, database, username, password, pooling, port);

            // Impalah - Gets the dao factory
            _daoFactory = new NHibernateDaoFactory();

            // Get DAO for asset
            _assetsDao = _daoFactory.GetAssetsDao();


            // TODO - check table existence and create with hibernate ddl
//            TestTables();



        }

        public string Version
        {
//            get { return _dbConnection.getVersion(); }
            get { return "1.0"; }
        }

        public string Name
        {
            get { return "Hibernate Asset storage engine"; }
        }

        #endregion
    }
}
