/*
* Copyright (c) Contributors, http://opensimulator.org/
* See CONTRIBUTORS.TXT for a full list of copyright holders.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions are met:
*     * Redistributions of source code must retain the above copyright
*       notice, this list of conditions and the following disclaimer.
*     * Redistributions in binary form must reproduce the above copyright
*       notice, this list of conditions and the following disclaimer in the
*       documentation and/or other materials provided with the distribution.
*     * Neither the name of the OpenSim Project nor the
*       names of its contributors may be used to endorse or promote products
*       derived from this software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE DEVELOPERS ``AS IS'' AND ANY
* EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
* DISCLAIMED. IN NO EVENT SHALL THE CONTRIBUTORS BE LIABLE FOR ANY
* DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
* LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
* ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
* (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
* 
*/
using System;
using System.Collections.Generic;
using System.Data;
using System.Text.RegularExpressions;
using libsecondlife;
using OpenSim.Framework.Console;

using Opensim.Framework.Persistence;
using Opensim.Framework.Persistence.mappings;
using Opensim.Framework.Persistence.interfaces;

using NHibernate;



namespace OpenSim.Framework.Data.Hibernate
{
    /// <summary>
    /// A database interface class to a user profile storage system
    /// </summary>
    internal class UserData : IUserData
    {
        /// <summary>
        /// Database manager for MySQL
        /// </summary>
        public Manager database;

        private IDaoFactory _daoFactory;

        private IUsersDao _usersDao;
        private IAgentsDao _agentsDao;



        /// <summary>
        /// Loads and initialises the MySQL storage plugin
        /// </summary>
        public void Initialise()
        {
            //// Load from an INI file connection details
            //// TODO: move this to XML?
            //IniFile GridDataMySqlFile = new IniFile("mysql_connection.ini");
            //string settingHostname = GridDataMySqlFile.ParseFileReadValue("hostname");
            //string settingDatabase = GridDataMySqlFile.ParseFileReadValue("database");
            //string settingUsername = GridDataMySqlFile.ParseFileReadValue("username");
            //string settingPassword = GridDataMySqlFile.ParseFileReadValue("password");
            //string settingPooling = GridDataMySqlFile.ParseFileReadValue("pooling");
            //string settingPort = GridDataMySqlFile.ParseFileReadValue("port");

            //database =
            //    new Manager(settingHostname, settingDatabase, settingUsername, settingPassword, settingPooling,
            //                     settingPort);

            // Impalah - Gets the dao factory
            _daoFactory = new NHibernateDaoFactory();

            // Get DAO for Inventory folders and items
            _usersDao = _daoFactory.GetUsersDao();
            _agentsDao = _daoFactory.GetAgentsDao();

            // TODO - Create with hibernate ddl
            //            TestTables();
        }

        #region Test and initialization code

        /// <summary>
        /// Ensure that the user related tables exists and are at the latest version
        /// </summary>
        private void TestTables()
        {
            Dictionary<string, string> tableList = new Dictionary<string, string>();

            tableList["agents"] = null;
            tableList["users"] = null;
            tableList["userfriends"] = null;
            database.GetTableVersion(tableList);

            UpgradeAgentsTable(tableList["agents"]);
            UpgradeUsersTable(tableList["users"]);
            UpgradeFriendsTable(tableList["userfriends"]);

        }

        /// <summary>
        /// Create or upgrade the table if necessary
        /// </summary>
        /// <param name="oldVersion">A null indicates that the table does not
        /// currently exist</param>
        private void UpgradeAgentsTable(string oldVersion)
        {
            // null as the version, indicates that the table didn't exist
            if (oldVersion == null)
            {
                database.ExecuteResourceSql("CreateAgentsTable.sql");
                return;
            }
        }

        /// <summary>
        /// Create or upgrade the table if necessary
        /// </summary>
        /// <param name="oldVersion">A null indicates that the table does not
        /// currently exist</param>
        private void UpgradeUsersTable(string oldVersion)
        {
            // null as the version, indicates that the table didn't exist
            if (oldVersion == null)
            {
                database.ExecuteResourceSql("CreateUsersTable.sql");
                return;
            }
            else if (oldVersion.Contains("Rev. 1"))
            {
                database.ExecuteResourceSql("UpgradeUsersTableToVersion2.sql");
                return;
            }
            //MainLog.Instance.Verbose("DB","DBVers:" + oldVersion);
        }

        /// <summary>
        /// Create or upgrade the table if necessary
        /// </summary>
        /// <param name="oldVersion">A null indicates that the table does not
        /// currently exist</param>
        private void UpgradeFriendsTable(string oldVersion)
        {
            // null as the version, indicates that the table didn't exist
            if (oldVersion == null)
            {
                database.ExecuteResourceSql("CreateUserFriendsTable.sql");
                return;
            }
        }

        #endregion

        // see IUserData
        public UserProfileData GetUserByName(string user, string last)
        {

            // Select in Db for name/lastname
            Users usrFilter = new Users();
            // usrFilter.Uuid = LLUUID.Zero.ToString();
            usrFilter.Username = user;
            usrFilter.Lastname = last;
            // Establish filter
            // The fields to exclude are INDEXES in DB, not only Ids
            string[] fields = { "Uuid", "Created", "LastLogin", "ProfileCanDoMask", "ProfileWantDoMask", "HomeRegion" };

            // Do the search
            try
            {
                Users usr = _usersDao.GetUniqueByExample(usrFilter, fields);

                if (usr != null)
                {
                    // Map results to UserProfileData
                    // TODO - Change to mapping class
                    UserProfileData retval = new UserProfileData();

                    retval.UUID = new LLUUID(usr.Uuid);
                    retval.username = usr.Username;
                    retval.surname = usr.Lastname;

                    retval.passwordHash = usr.PasswordHash;
                    retval.passwordSalt = usr.PasswordSalt;

                    retval.homeRegion = (UInt64)usr.HomeRegion;
                    retval.homeLocation = new LLVector3(
                        (float)usr.HomeLocationx,
                        (float)usr.HomeLocationy,
                        (float)usr.HomeLocationz);
                    retval.homeLookAt = new LLVector3(
                        (float)usr.HomeLookAtx,
                        (float)usr.HomeLookAty,
                        (float)usr.HomeLookAtz);

                    retval.created = (int)usr.Created;
                    retval.lastLogin = (int)usr.LastLogin;

                    retval.userInventoryURI = usr.UserInventoryuri;
                    retval.userAssetURI = usr.UserAsseturi;

                    retval.profileCanDoMask = (UInt32)usr.ProfileCanDoMask;
                    retval.profileWantDoMask = (UInt32)usr.ProfileWantDoMask;

                    retval.profileAboutText = usr.ProfileAboutText;
                    retval.profileFirstText = usr.ProfileFirstText;

                    retval.profileImage = new LLUUID(usr.ProfileImage);
                    retval.profileFirstImage = new LLUUID(usr.ProfileFirstImage);

                    if (usr.WebLoginKey == null)
                    {
                        retval.webLoginKey = LLUUID.Zero;
                    }
                    else
                    {
                        retval.webLoginKey = new LLUUID(usr.WebLoginKey);
                    }

                    return retval;

                }
                else
                {
                    return null;
                }






            }
            catch (NHibernate.ObjectNotFoundException)
            {
                // TODO - What to do with exception?
                return null;
            }






            //try
            //{
            //    lock (database)
            //    {
            //        Dictionary<string, string> param = new Dictionary<string, string>();
            //        param["?first"] = user;
            //        param["?second"] = last;

            //        IDbCommand result =
            //            database.Query("SELECT * FROM users WHERE username = ?first AND lastname = ?second", param);
            //        IDataReader reader = result.ExecuteReader();

            //        UserProfileData row = database.readUserRow(reader);

            //        reader.Close();
            //        result.Dispose();
            //        return row;
            //    }
            //}
            //catch (Exception e)
            //{
            //    database.Reconnect();
            //    MainLog.Instance.Error(e.ToString());
            //    return null;
            //}
        }

        #region User Friends List Data

        public void AddNewUserFriend(LLUUID friendlistowner, LLUUID friend, uint perms)
        {
            int dtvalue = Util.UnixTimeSinceEpoch();

            Dictionary<string, string> param = new Dictionary<string, string>();
            param["?ownerID"] = friendlistowner.UUID.ToString();
            param["?friendID"] = friend.UUID.ToString();
            param["?friendPerms"] = perms.ToString();
            param["?datetimestamp"] = dtvalue.ToString();
            
            try 
            {
                lock (database)
                {
                    IDbCommand adder =
                        database.Query(
                        "INSERT INTO `userfriends` " +
                        "(`ownerID`,`friendID`,`friendPerms`,`datetimestamp`) " + 
                        "VALUES " +
                        "(?ownerID,?friendID,?friendPerms,?datetimestamp)",
                            param);
                    adder.ExecuteNonQuery();

                    adder =
                        database.Query(
                        "INSERT INTO `userfriends` " +
                        "(`ownerID`,`friendID`,`friendPerms`,`datetimestamp`) " +
                        "VALUES " +
                        "(?friendID,?ownerID,?friendPerms,?datetimestamp)",
                            param);
                    adder.ExecuteNonQuery();

                }
            }
            catch (Exception e)
            {
                database.Reconnect();
                MainLog.Instance.Error(e.ToString());
                return;
            }
        }

        public void RemoveUserFriend(LLUUID friendlistowner, LLUUID friend)
        {
            Dictionary<string, string> param = new Dictionary<string, string>();
            param["?ownerID"] = friendlistowner.UUID.ToString();
            param["?friendID"] = friend.UUID.ToString();


            try
            {
                lock (database)
                {
                    IDbCommand updater =
                        database.Query(
                        "delete from userfriends " +
                        "where ownerID = ?ownerID and friendID = ?friendID",
                            param);
                    updater.ExecuteNonQuery();

                    updater =
                        database.Query(
                        "delete from userfriends " +
                        "where ownerID = ?friendID and friendID = ?ownerID",
                            param);
                    updater.ExecuteNonQuery();

                }
            }
            catch (Exception e)
            {
                database.Reconnect();
                MainLog.Instance.Error(e.ToString());
                return;
            }
        }
        public void UpdateUserFriendPerms(LLUUID friendlistowner, LLUUID friend, uint perms)
        {
            Dictionary<string, string> param = new Dictionary<string, string>();
            param["?ownerID"] = friendlistowner.UUID.ToString();
            param["?friendID"] = friend.UUID.ToString();
            param["?friendPerms"] = perms.ToString();
            

            try
            {
                lock (database)
                {
                    IDbCommand updater =
                        database.Query(
                        "update userfriends " +
                        "SET friendPerms = ?friendPerms " +
                        "where ownerID = ?ownerID and friendID = ?friendID",
                            param);
                    updater.ExecuteNonQuery();

                }
            }
            catch (Exception e)
            {
                database.Reconnect();
                MainLog.Instance.Error(e.ToString());
                return;
            }
        }


        public List<FriendListItem> GetUserFriendList(LLUUID friendlistowner)
        {
            List<FriendListItem> Lfli = new List<FriendListItem>();

            Dictionary<string, string> param = new Dictionary<string, string>();
            param["?ownerID"] = friendlistowner.UUID.ToString();

            try
            {
                lock (database)
                {
                    //Left Join userfriends to itself
                    IDbCommand result =
                        database.Query(
                        "select a.ownerID,a.friendID,a.friendPerms,b.friendPerms as ownerperms from userfriends as a, userfriends as b" +
                        " where a.ownerID = ?ownerID and b.ownerID = a.friendID and b.friendID = a.ownerID",
                            param);
                    IDataReader reader = result.ExecuteReader();


                    while (reader.Read())
                    {
                        FriendListItem fli = new FriendListItem();
                        fli.FriendListOwner = new LLUUID((string)reader["ownerID"]);
                        fli.Friend = new LLUUID((string)reader["friendID"]);
                        fli.FriendPerms = (uint)Convert.ToInt32(reader["friendPerms"]);

                        // This is not a real column in the database table, it's a joined column from the opposite record
                        fli.FriendListOwnerPerms = (uint)Convert.ToInt32(reader["ownerperms"]);
                        
                        Lfli.Add(fli);
                    }
                    reader.Close();
                    result.Dispose();
                }
            }
            catch (Exception e)
            {
                database.Reconnect();
                MainLog.Instance.Error(e.ToString());
                return Lfli;
            }

            return Lfli;
        }

        #endregion

        public void UpdateUserCurrentRegion(LLUUID avatarid, LLUUID regionuuid)
        {
            MainLog.Instance.Verbose("USER", "Stub UpdateUserCUrrentRegion called");
        }


        public List<Framework.AvatarPickerAvatar> GeneratePickerResults(LLUUID queryID, string query)
        {
            List<Framework.AvatarPickerAvatar> returnlist = new List<Framework.AvatarPickerAvatar>();

            Regex objAlphaNumericPattern = new Regex("[^a-zA-Z0-9]");

            string[] querysplit;
            querysplit = query.Split(' ');
            if (querysplit.Length == 2)
            {
                Dictionary<string, string> param = new Dictionary<string, string>();
                param["?first"] = objAlphaNumericPattern.Replace(querysplit[0], String.Empty) + "%";
                param["?second"] = objAlphaNumericPattern.Replace(querysplit[1], String.Empty) + "%";
                try
                {
                    lock (database)
                    {
                        IDbCommand result =
                            database.Query(
                                "SELECT UUID,username,lastname FROM users WHERE username like ?first AND lastname like ?second LIMIT 100",
                                param);
                        IDataReader reader = result.ExecuteReader();


                        while (reader.Read())
                        {
                            Framework.AvatarPickerAvatar user = new Framework.AvatarPickerAvatar();
                            user.AvatarID = new LLUUID((string) reader["UUID"]);
                            user.firstName = (string) reader["username"];
                            user.lastName = (string) reader["lastname"];
                            returnlist.Add(user);
                        }
                        reader.Close();
                        result.Dispose();
                    }
                }
                catch (Exception e)
                {
                    database.Reconnect();
                    MainLog.Instance.Error(e.ToString());
                    return returnlist;
                }
            }
            else if (querysplit.Length == 1)
            {
                try
                {
                    lock (database)
                    {
                        Dictionary<string, string> param = new Dictionary<string, string>();
                        param["?first"] = objAlphaNumericPattern.Replace(querysplit[0], String.Empty) + "%";

                        IDbCommand result =
                            database.Query(
                                "SELECT UUID,username,lastname FROM users WHERE username like ?first OR lastname like ?first LIMIT 100",
                                param);
                        IDataReader reader = result.ExecuteReader();


                        while (reader.Read())
                        {
                            Framework.AvatarPickerAvatar user = new Framework.AvatarPickerAvatar();
                            user.AvatarID = new LLUUID((string) reader["UUID"]);
                            user.firstName = (string) reader["username"];
                            user.lastName = (string) reader["lastname"];
                            returnlist.Add(user);
                        }
                        reader.Close();
                        result.Dispose();
                    }
                }
                catch (Exception e)
                {
                    database.Reconnect();
                    MainLog.Instance.Error(e.ToString());
                    return returnlist;
                }
            }
            return returnlist;
        }

        // see IUserData
        public UserProfileData GetUserByUUID(LLUUID uuid)
        {

            try
            {
                Users usr = _usersDao.GetById(uuid.ToString(), true);

                // Map results to UserProfileData
                // TODO - Change to mapping class
                UserProfileData retval = new UserProfileData();

                retval.UUID = new LLUUID(usr.Uuid);
                retval.username = usr.Username;
                retval.surname = usr.Lastname;

                retval.passwordHash = usr.PasswordHash;
                retval.passwordSalt = usr.PasswordSalt;

                retval.homeRegion = (UInt64)usr.HomeRegion;
                retval.homeLocation = new LLVector3(
                    (float)usr.HomeLocationx,
                    (float)usr.HomeLocationy,
                    (float)usr.HomeLocationz);
                retval.homeLookAt = new LLVector3(
                    (float)usr.HomeLookAtx,
                    (float)usr.HomeLookAty,
                    (float)usr.HomeLookAtz);

                retval.created = (int)usr.Created;
                retval.lastLogin = (int)usr.LastLogin;

                retval.userInventoryURI = usr.UserInventoryuri;
                retval.userAssetURI = usr.UserAsseturi;

                retval.profileCanDoMask = (UInt32)usr.ProfileCanDoMask;
                retval.profileWantDoMask = (UInt32)usr.ProfileWantDoMask;

                retval.profileAboutText = usr.ProfileAboutText;
                retval.profileFirstText = usr.ProfileFirstText;

                retval.profileImage = new LLUUID(usr.ProfileImage);
                retval.profileFirstImage = new LLUUID(usr.ProfileFirstImage);

                if (usr.WebLoginKey == null)
                {
                    retval.webLoginKey = LLUUID.Zero;
                }
                else
                {
                    retval.webLoginKey = new LLUUID(usr.WebLoginKey);
                }

                return retval;




            }
            catch (NHibernate.ObjectNotFoundException nfe)
            {
                return null;
            }




            //try
            //{
            //    lock (database)
            //    {
            //        Dictionary<string, string> param = new Dictionary<string, string>();
            //        param["?uuid"] = uuid.ToString();

            //        IDbCommand result = database.Query("SELECT * FROM users WHERE UUID = ?uuid", param);
            //        IDataReader reader = result.ExecuteReader();

            //        UserProfileData row = database.readUserRow(reader);

            //        reader.Close();
            //        result.Dispose();

            //        return row;
            //    }
            //}
            //catch (Exception e)
            //{
            //    database.Reconnect();
            //    MainLog.Instance.Error(e.ToString());
            //    return null;
            //}
        }

        /// <summary>
        /// Returns a user session searching by name
        /// </summary>
        /// <param name="name">The account name</param>
        /// <returns>The users session</returns>
        public UserAgentData GetAgentByName(string name)
        {
            return GetAgentByName(name.Split(' ')[0], name.Split(' ')[1]);
        }

        /// <summary>
        /// Returns a user session by account name
        /// </summary>
        /// <param name="user">First part of the users account name</param>
        /// <param name="last">Second part of the users account name</param>
        /// <returns>The users session</returns>
        public UserAgentData GetAgentByName(string user, string last)
        {
            UserProfileData profile = GetUserByName(user, last);
            return GetAgentByUUID(profile.UUID);
        }

        public void StoreWebLoginKey(LLUUID AgentID, LLUUID WebLoginKey)
        {
            
            Dictionary<string, string> param = new Dictionary<string, string>();
            param["?UUID"] = AgentID.UUID.ToString();
            param["?webLoginKey"] = WebLoginKey.UUID.ToString();

            try
            {
                lock (database)
                {
                    IDbCommand updater =
                        database.Query(
                        "update users " +
                        "SET webLoginKey = ?webLoginKey " +
                        "where UUID = ?UUID",
                            param);
                    updater.ExecuteNonQuery();

                }
            }
            catch (Exception e)
            {
                database.Reconnect();
                MainLog.Instance.Error(e.ToString());
                return;
            }

                
            

        }

        /// <summary>
        /// Returns an agent session by account UUID
        /// </summary>
        /// <param name="uuid">The accounts UUID</param>
        /// <returns>The users session</returns>
        public UserAgentData GetAgentByUUID(LLUUID uuid)
        {

            try
            {
                Agents agent = _agentsDao.GetById(uuid.ToString(), true);

                // Agent returned, fill
                UserAgentData retval = new UserAgentData();

                // Agent IDs
                retval.UUID = new LLUUID(agent.Uuid);
                retval.sessionID = new LLUUID(agent.Sessionid);
                retval.secureSessionID = new LLUUID(agent.SecureSessionid);

                // Agent Who?
                retval.agentIP = agent.Agentip;
                retval.agentPort = (UInt32)agent.AgentPort;
                retval.agentOnline = (agent.AgentOnline == 0 ? false : true);

                // Login/Logout times (UNIX Epoch)
                retval.loginTime = (Int32)agent.LoginTime;
                retval.logoutTime = (Int32)agent.LogoutTime;

                // Current position
                retval.currentRegion = agent.CurrentRegion;
                retval.currentHandle = (UInt64)agent.CurrentHandle;
                LLVector3.TryParse(agent.CurrentPos, out retval.currentPos);

                return retval;

            } catch (NHibernate.ObjectNotFoundException nfe)
            {
                // If does not exists...
                return null;
            }




            //try
            //{
            //    lock (database)
            //    {
            //        Dictionary<string, string> param = new Dictionary<string, string>();
            //        param["?uuid"] = uuid.ToString();

            //        IDbCommand result = database.Query("SELECT * FROM agents WHERE UUID = ?uuid", param);
            //        IDataReader reader = result.ExecuteReader();

            //        UserAgentData row = database.readAgentRow(reader);

            //        reader.Close();
            //        result.Dispose();

            //        return row;
            //    }
            //}
            //catch (Exception e)
            //{
            //    database.Reconnect();
            //    MainLog.Instance.Error(e.ToString());
            //    return null;
            //}
        }

        /// <summary>
        /// Creates a new users profile
        /// </summary>
        /// <param name="user">The user profile to create</param>
        public void AddNewUserProfile(UserProfileData user)
        {

            // TODO - Database lock needed???
            // TODO - Why a replace??? Easy method for insertOrUpdate - If exists, update, if not, insert

            // Impalah - Using Hibernate

            // Create asset
            Users dbuser;
            // Object does not exist
            dbuser = new Users();
            dbuser.Uuid = user.UUID.ToString();


            dbuser.Username = user.username;
            dbuser.Lastname = user.surname;
            dbuser.PasswordHash = user.passwordHash;
            dbuser.PasswordSalt = user.passwordSalt;
            dbuser.HomeRegion = (long)user.homeRegion;
            dbuser.HomeLocationx = user.homeLocation.X;
            dbuser.HomeLocationy = user.homeLocation.Y;
            dbuser.HomeLocationz = user.homeLocation.Z;
            dbuser.HomeLookAtx = user.homeLookAt.X;
            dbuser.HomeLookAty = user.homeLookAt.Y;
            dbuser.HomeLookAtz = user.homeLookAt.Z;
            dbuser.Created = user.created;
            dbuser.LastLogin = user.lastLogin;
            dbuser.UserInventoryuri = user.userInventoryURI;
            dbuser.UserAsseturi = user.userAssetURI;

            // TODO - What???
            dbuser.ProfileCanDoMask = 0;
            dbuser.ProfileWantDoMask = 0;
            dbuser.ProfileAboutText = String.Empty;
            dbuser.ProfileFirstText = String.Empty;
            dbuser.ProfileImage = LLUUID.Zero.ToString();
            dbuser.ProfileFirstImage = LLUUID.Zero.ToString();
            dbuser.WebLoginKey = LLUUID.Random().ToString();

            // Save or update
            _usersDao.Save(dbuser);

            // TODO - Transaction management
            _usersDao.CommitChanges();


            //try
            //{
            //    lock (database)
            //    {
            //        database.insertUserRow(user.UUID, user.username, user.surname, user.passwordHash, user.passwordSalt,
            //                               user.homeRegion, user.homeLocation.X, user.homeLocation.Y,
            //                               user.homeLocation.Z,
            //                               user.homeLookAt.X, user.homeLookAt.Y, user.homeLookAt.Z, user.created,
            //                               user.lastLogin, user.userInventoryURI, user.userAssetURI,
            //                               user.profileCanDoMask, user.profileWantDoMask,
            //                               user.profileAboutText, user.profileFirstText, user.profileImage,
            //                               user.profileFirstImage, user.webLoginKey);
            //    }
            //}
            //catch (Exception e)
            //{
            //    database.Reconnect();
            //    MainLog.Instance.Error(e.ToString());
            //}
        }

        /// <summary>
        /// Creates a new agent
        /// </summary>
        /// <param name="agent">The agent to create</param>
        public void AddNewUserAgent(UserAgentData agent)
        {
            // Do nothing.
        }

        /// <summary>
        /// Updates a user profile stored in the DB
        /// </summary>
        /// <param name="user">The profile data to use to update the DB</param>
        public bool UpdateUserProfile(UserProfileData user)
        {
            // TODO: implement
            return true;
        }

        /// <summary>
        /// Performs a money transfer request between two accounts
        /// </summary>
        /// <param name="from">The senders account ID</param>
        /// <param name="to">The receivers account ID</param>
        /// <param name="amount">The amount to transfer</param>
        /// <returns>Success?</returns>
        public bool MoneyTransferRequest(LLUUID from, LLUUID to, uint amount)
        {
            return false;
        }

        /// <summary>
        /// Performs an inventory transfer request between two accounts
        /// </summary>
        /// <remarks>TODO: Move to inventory server</remarks>
        /// <param name="from">The senders account ID</param>
        /// <param name="to">The receivers account ID</param>
        /// <param name="item">The item to transfer</param>
        /// <returns>Success?</returns>
        public bool InventoryTransferRequest(LLUUID from, LLUUID to, LLUUID item)
        {
            return false;
        }

        /// <summary>
        /// Database provider name
        /// </summary>
        /// <returns>Provider name</returns>
        public string getName()
        {
            return "Hibernate Userdata Interface";
        }

        /// <summary>
        /// Database provider version
        /// </summary>
        /// <returns>provider version</returns>
        public string GetVersion()
        {
            return "0.1";
        }
    }
}